/****************************************************************/
/* Tiny Z80 Disassembler					*/
/*						Ver 0.1		*/
/*					(C) Showzoh Fukunaga	*/
/****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#ifndef TRUE
#define TRUE (1)
#define FALSE (0)
#endif



/* option-file  1Ԥκʸ */
#define OPTION_MAX_CHARS	(255)





/*
  ڥɤη
     NOTHING	  ʤ	   XX		 Τޤɽ
     NUM_8	  8bit   XX nn	 nn 16ʤɽ
     NUM_16	  16bit  XX nn nn	 nn nn 16ʤɽ
     MEMORY	  	   XX nn nn	 nn nn 16ʤɽ
     ADR_REL	   XX nn	 Ϥ٥ɽ
     ADR_ABS	   XX nn nn	 Ϥ٥ɽ
     PREFIX
       NOTHING	  ʤ	   XX XX	 Τޤɽ
       NUM_8	  8bit   XX XX nn	 nn 16ʤɽ
       NUM_16	  16bit  XX XX nn nn	 nn nn 16ʤɽ
       MEMORY	  	   XX XX nn nn	 nn nn 16ʤɽ
       INDEX	  	   XX XX nn	 nn  10ʤɽ
       IDX_NUM	  &  XX XX nn mm	 nn  10ʡmm 16ʤɽ
       UNEXIST	  ̤   XX XX	 XX XX 16ʤɽ
       SKIP	  ̤   XX		 XX 16ʤɽ
       PREFIX
         INDEX         XX XX nn XX	 nn  10ʤɽ
	 UNEXIST  ̤   XX XX XX XX	 XX XX XX XX 16ʤɽ
*/


enum{
  OP_NOTHING,
  OP_NUM_8,
  OP_NUM_16,
  OP_MEMORY,
  OP_ADR_ABS,
  OP_ADR_REL,
  OP_PREFIX,
  OP_INDEX,
  OP_IDX_NUM,
  OP_UNEXIST,
  OP_SKIP,
};


typedef struct{
  int		type;
  char		*str;
} Mnemonics;



Mnemonics	Instruction[256] =
{
  OP_NOTHING,	" NOP",			/* 00 */
  OP_NUM_16,	" LD	BC,0%04XH",
  OP_NOTHING,	" LD	(BC),A",
  OP_NOTHING,	" INC	BC",
  OP_NOTHING,	" INC	B",
  OP_NOTHING,	" DEC	B",
  OP_NUM_8,	" LD	B,0%02XH",
  OP_NOTHING,	" RLCA",

  OP_NOTHING,	" EX	AF,AF'",	/* 08 */
  OP_NOTHING,	" ADD	HL,BC",
  OP_NOTHING,	" LD	A,(BC)",
  OP_NOTHING,	" DEC	BC",
  OP_NOTHING,	" INC	C",
  OP_NOTHING,	" DEC	C",
  OP_NUM_8,	" LD	C,0%02XH",
  OP_NOTHING,	" RRCA",

  OP_ADR_REL,	" DJNZ	%s",		/* 10 */
  OP_NUM_16,	" LD	DE,0%04XH",
  OP_NOTHING,	" LD	(DE),A",
  OP_NOTHING,	" INC	DE",
  OP_NOTHING,	" INC	D",
  OP_NOTHING,	" DEC	D",
  OP_NUM_8,	" LD	D,0%02XH",
  OP_NOTHING,	" RLA",

  OP_ADR_REL,	" JR	%s",		/* 18 */
  OP_NOTHING,	" ADD	HL,DE",
  OP_NOTHING,	" LD	A,(DE)",
  OP_NOTHING,	" DEC	DE",
  OP_NOTHING,	" INC	E",
  OP_NOTHING,	" DEC	E",
  OP_NUM_8,	" LD	E,0%02XH",
  OP_NOTHING,	" RRA",

  OP_ADR_REL,	" JR	NZ,%s",		/* 20 */
  OP_NUM_16,	" LD	HL,0%04XH",
  OP_MEMORY,	" LD	(0%04XH),HL",
  OP_NOTHING,	" INC	HL",
  OP_NOTHING,	" INC	H",
  OP_NOTHING,	" DEC	H",
  OP_NUM_8,	" LD	H,0%02XH",
  OP_NOTHING,	" DAA",

  OP_ADR_REL,	" JR	Z,%s",		/* 28 */
  OP_NOTHING,	" ADD	HL,HL",
  OP_MEMORY,	" LD	HL,(0%04XH)",
  OP_NOTHING,	" DEC	HL",
  OP_NOTHING,	" INC	L",
  OP_NOTHING,	" DEC	L",
  OP_NUM_8,	" LD	L,0%02XH",
  OP_NOTHING,	" CPL",

  OP_ADR_REL,	" JR	NC,%s",		/* 30 */
  OP_NUM_16,	" LD	SP,0%04XH",
  OP_MEMORY,	" LD	(0%04XH),A",
  OP_NOTHING,	" INC	SP",
  OP_NOTHING,	" INC	(HL)",
  OP_NOTHING,	" DEC	(HL)",
  OP_NUM_8,	" LD	(HL),0%02XH",
  OP_NOTHING,	" SCF",

  OP_ADR_REL,	" JR	C,%s",		/* 38 */
  OP_NOTHING,	" ADD	HL,SP",
  OP_MEMORY,	" LD	A,(0%04XH)",
  OP_NOTHING,	" DEC	SP",
  OP_NOTHING,	" INC	A",
  OP_NOTHING,	" DEC	A",
  OP_NUM_8,	" LD	A,0%02XH",
  OP_NOTHING,	" CCF",

  OP_NOTHING,	" LD	B,B",		/* 40 */
  OP_NOTHING,	" LD	B,C",
  OP_NOTHING,	" LD	B,D",
  OP_NOTHING,	" LD	B,E",
  OP_NOTHING,	" LD	B,H",
  OP_NOTHING,	" LD	B,L",
  OP_NOTHING,	" LD	B,(HL)",
  OP_NOTHING,	" LD	B,A",

  OP_NOTHING,	" LD	C,B",		/* 48 */
  OP_NOTHING,	" LD	C,C",
  OP_NOTHING,	" LD	C,D",
  OP_NOTHING,	" LD	C,E",
  OP_NOTHING,	" LD	C,H",
  OP_NOTHING,	" LD	C,L",
  OP_NOTHING,	" LD	C,(HL)",
  OP_NOTHING,	" LD	C,A",

  OP_NOTHING,	" LD	D,B",		/* 50 */
  OP_NOTHING,	" LD	D,C",
  OP_NOTHING,	" LD	D,D",
  OP_NOTHING,	" LD	D,E",
  OP_NOTHING,	" LD	D,H",
  OP_NOTHING,	" LD	D,L",
  OP_NOTHING,	" LD	D,(HL)",
  OP_NOTHING,	" LD	D,A",

  OP_NOTHING,	" LD	E,B",		/* 58 */
  OP_NOTHING,	" LD	E,C",
  OP_NOTHING,	" LD	E,D",
  OP_NOTHING,	" LD	E,E",
  OP_NOTHING,	" LD	E,H",
  OP_NOTHING,	" LD	E,L",
  OP_NOTHING,	" LD	E,(HL)",
  OP_NOTHING,	" LD	E,A",

  OP_NOTHING,	" LD	H,B",		/* 60 */
  OP_NOTHING,	" LD	H,C",
  OP_NOTHING,	" LD	H,D",
  OP_NOTHING,	" LD	H,E",
  OP_NOTHING,	" LD	H,H",
  OP_NOTHING,	" LD	H,L",
  OP_NOTHING,	" LD	H,(HL)",
  OP_NOTHING,	" LD	H,A",

  OP_NOTHING,	" LD	L,B",		/* 68 */
  OP_NOTHING,	" LD	L,C",
  OP_NOTHING,	" LD	L,D",
  OP_NOTHING,	" LD	L,E",
  OP_NOTHING,	" LD	L,H",
  OP_NOTHING,	" LD	L,L",
  OP_NOTHING,	" LD	L,(HL)",
  OP_NOTHING,	" LD	L,A",

  OP_NOTHING,	" LD	(HL),B",	/* 70 */
  OP_NOTHING,	" LD	(HL),C",
  OP_NOTHING,	" LD	(HL),D",
  OP_NOTHING,	" LD	(HL),E",
  OP_NOTHING,	" LD	(HL),H",
  OP_NOTHING,	" LD	(HL),L",
  OP_NOTHING,	" HALT",
  OP_NOTHING,	" LD	(HL),A",

  OP_NOTHING,	" LD	A,B",		/* 78 */
  OP_NOTHING,	" LD	A,C",
  OP_NOTHING,	" LD	A,D",
  OP_NOTHING,	" LD	A,E",
  OP_NOTHING,	" LD	A,H",
  OP_NOTHING,	" LD	A,L",
  OP_NOTHING,	" LD	A,(HL)",
  OP_NOTHING,	" LD	A,A",

  OP_NOTHING,	" ADD	A,B",		/* 80 */
  OP_NOTHING,	" ADD	A,C",
  OP_NOTHING,	" ADD	A,D",
  OP_NOTHING,	" ADD	A,E",
  OP_NOTHING,	" ADD	A,H",
  OP_NOTHING,	" ADD	A,L",
  OP_NOTHING,	" ADD	A,(HL)",
  OP_NOTHING,	" ADD	A,A",

  OP_NOTHING,	" ADC	A,B",		/* 88 */
  OP_NOTHING,	" ADC	A,C",
  OP_NOTHING,	" ADC	A,D",
  OP_NOTHING,	" ADC	A,E",
  OP_NOTHING,	" ADC	A,H",
  OP_NOTHING,	" ADC	A,L",
  OP_NOTHING,	" ADC	A,(HL)",
  OP_NOTHING,	" ADC	A,A",

  OP_NOTHING,	" SUB	B",		/* 90 */
  OP_NOTHING,	" SUB	C",
  OP_NOTHING,	" SUB	D",
  OP_NOTHING,	" SUB	E",
  OP_NOTHING,	" SUB	H",
  OP_NOTHING,	" SUB	L",
  OP_NOTHING,	" SUB	(HL)",
  OP_NOTHING,	" SUB	A",

  OP_NOTHING,	" SBC	A,B",		/* 98 */
  OP_NOTHING,	" SBC	A,C",
  OP_NOTHING,	" SBC	A,D",
  OP_NOTHING,	" SBC	A,E",
  OP_NOTHING,	" SBC	A,H",
  OP_NOTHING,	" SBC	A,L",
  OP_NOTHING,	" SBC	A,(HL)",
  OP_NOTHING,	" SBC	A,A",

  OP_NOTHING,	" AND	B",		/* A0 */
  OP_NOTHING,	" AND	C",
  OP_NOTHING,	" AND	D",
  OP_NOTHING,	" AND	E",
  OP_NOTHING,	" AND	H",
  OP_NOTHING,	" AND	L",
  OP_NOTHING,	" AND	(HL)",
  OP_NOTHING,	" AND	A",

  OP_NOTHING,	" XOR	B",		/* A8 */
  OP_NOTHING,	" XOR	C",
  OP_NOTHING,	" XOR	D",
  OP_NOTHING,	" XOR	E",
  OP_NOTHING,	" XOR	H",
  OP_NOTHING,	" XOR	L",
  OP_NOTHING,	" XOR	(HL)",
  OP_NOTHING,	" XOR	A",

  OP_NOTHING,	" OR	B",		/* B0 */
  OP_NOTHING,	" OR	C",
  OP_NOTHING,	" OR	D",
  OP_NOTHING,	" OR	E",
  OP_NOTHING,	" OR	H",
  OP_NOTHING,	" OR	L",
  OP_NOTHING,	" OR	(HL)",
  OP_NOTHING,	" OR	A",

  OP_NOTHING,	" CP	B",		/* B8 */
  OP_NOTHING,	" CP	C",
  OP_NOTHING,	" CP	D",
  OP_NOTHING,	" CP	E",
  OP_NOTHING,	" CP	H",
  OP_NOTHING,	" CP	L",
  OP_NOTHING,	" CP	(HL)",
  OP_NOTHING,	" CP	A",

  OP_NOTHING,	" RET	NZ",		/* C0 */
  OP_NOTHING,	" POP	BC",
  OP_ADR_ABS,	" JP	NZ,%s",
  OP_ADR_ABS,	" JP	%s",
  OP_ADR_ABS,	" CALL	NZ,%s",
  OP_NOTHING,	" PUSH	BC",
  OP_NUM_8,	" ADD	A,0%02XH",
  OP_NOTHING,	" RST	00H",

  OP_NOTHING,	" RET	Z",		/* C8 */
  OP_NOTHING,	" RET",
  OP_ADR_ABS,	" JP	Z,%s",
  OP_PREFIX,	NULL,
  OP_ADR_ABS,	" CALL	Z,%s",
  OP_ADR_ABS,	" CALL	%s",
  OP_NUM_8,	" ADC	A,0%02XH",
  OP_NOTHING,	" RST	08H",

  OP_NOTHING,	" RET	NC",		/* D0 */
  OP_NOTHING,	" POP	DE",
  OP_ADR_ABS,	" JP	NC,%s",
  OP_NUM_8,	" OUT	(0%02XH),A",
  OP_ADR_ABS,	" CALL	NC,%s",
  OP_NOTHING,	" PUSH	DE",
  OP_NUM_8,	" SUB	0%02XH",
  OP_NOTHING,	" RST	10H",

  OP_NOTHING,	" RET	C",		/* D8 */
  OP_NOTHING,	" EXX",
  OP_ADR_ABS,	" JP	C,%s",
  OP_NUM_8,	" IN	A,(0%02XH)",
  OP_ADR_ABS,	" CALL	C,%s",
  OP_PREFIX,	NULL,
  OP_NUM_8,	" SBC	A,0%02XH",
  OP_NOTHING,	" RST	18H",

  OP_NOTHING,	" RET	PO",		/* E0 */
  OP_NOTHING,	" POP	HL",
  OP_ADR_ABS,	" JP	PO,%s",
  OP_NOTHING,	" EX	HL,(SP)",
  OP_ADR_ABS,	" CALL	PO,%s",
  OP_NOTHING,	" PUSH	HL",
  OP_NUM_8,	" AND	0%02XH",
  OP_NOTHING,	" RST	20H",

  OP_NOTHING,	" RET	PE",		/* E8 */
  OP_NOTHING,	" JP	(HL)",
  OP_ADR_ABS,	" JP	PE,%s",
  OP_NOTHING,	" EX	DE,HL",
  OP_ADR_ABS,	" CALL	PE,%s",
  OP_PREFIX,	NULL,
  OP_NUM_8,	" XOR	0%02XH",
  OP_NOTHING,	" RST	28H",

  OP_NOTHING,	" RET	P",		/* F0 */
  OP_NOTHING,	" POP	AF",
  OP_ADR_ABS,	" JP	P,%s",
  OP_NOTHING,	" DI",
  OP_ADR_ABS,	" CALL	P,%s",
  OP_NOTHING,	" PUSH	AF",
  OP_NUM_8,	" OR	0%02XH",
  OP_NOTHING,	" RST	30H",

  OP_NOTHING,	" RET	M",		/* F8 */
  OP_NOTHING,	" LD	SP,HL",
  OP_ADR_ABS,	" JP	M,%s",
  OP_NOTHING,	" EI",
  OP_ADR_ABS,	" CALL	M,%s",
  OP_PREFIX,	NULL,
  OP_NUM_8,	" CP	0%02XH",
  OP_NOTHING,	" RST	38H"
};

Mnemonics	Instruction_CB[256]=
{
  OP_NOTHING,	" RLC	B",		/* CB 00 */
  OP_NOTHING,	" RLC	C",
  OP_NOTHING,	" RLC	D",
  OP_NOTHING,	" RLC	E",
  OP_NOTHING,	" RLC	H",
  OP_NOTHING,	" RLC	L",
  OP_NOTHING,	" RLC	(HL)",
  OP_NOTHING,	" RLC	A",

  OP_NOTHING,	" RRC	B",		/* CB 08 */
  OP_NOTHING,	" RRC	C",
  OP_NOTHING,	" RRC	D",
  OP_NOTHING,	" RRC	E",
  OP_NOTHING,	" RRC	H",
  OP_NOTHING,	" RRC	L",
  OP_NOTHING,	" RRC	(HL)",
  OP_NOTHING,	" RRC	A",

  OP_NOTHING,	" RL	B",		/* CB 10 */
  OP_NOTHING,	" RL	C",
  OP_NOTHING,	" RL	D",
  OP_NOTHING,	" RL	E",
  OP_NOTHING,	" RL	H",
  OP_NOTHING,	" RL	L",
  OP_NOTHING,	" RL	(HL)",
  OP_NOTHING,	" RL	A",

  OP_NOTHING,	" RR	B",		/* CB 18 */
  OP_NOTHING,	" RR	C",
  OP_NOTHING,	" RR	D",
  OP_NOTHING,	" RR	E",
  OP_NOTHING,	" RR	H",
  OP_NOTHING,	" RR	L",
  OP_NOTHING,	" RR	(HL)",
  OP_NOTHING,	" RR	A",

  OP_NOTHING,	" SLA	B",		/* CB 20 */
  OP_NOTHING,	" SLA	C",
  OP_NOTHING,	" SLA	D",
  OP_NOTHING,	" SLA	E",
  OP_NOTHING,	" SLA	H",
  OP_NOTHING,	" SLA	L",
  OP_NOTHING,	" SLA	(HL)",
  OP_NOTHING,	" SLA	A",

  OP_NOTHING,	" SRA	B",		/* CB 28 */
  OP_NOTHING,	" SRA	C",
  OP_NOTHING,	" SRA	D",
  OP_NOTHING,	" SRA	E",
  OP_NOTHING,	" SRA	H",
  OP_NOTHING,	" SRA	L",
  OP_NOTHING,	" SRA	(HL)",
  OP_NOTHING,	" SRA	A",

  OP_NOTHING,	"+SLL	B",		/* CB 30 */
  OP_NOTHING,	"+SLL	C",
  OP_NOTHING,	"+SLL	D",
  OP_NOTHING,	"+SLL	E",
  OP_NOTHING,	"+SLL	H",
  OP_NOTHING,	"+SLL	L",
  OP_NOTHING,	"+SLL	(HL)",
  OP_NOTHING,	"+SLL	A",

  OP_NOTHING,	" SRL	B",		/* CB 38 */
  OP_NOTHING,	" SRL	C",
  OP_NOTHING,	" SRL	D",
  OP_NOTHING,	" SRL	E",
  OP_NOTHING,	" SRL	H",
  OP_NOTHING,	" SRL	L",
  OP_NOTHING,	" SRL	(HL)",
  OP_NOTHING,	" SRL	A",

  OP_NOTHING,	" BIT	0,B",		/* CB 40 */
  OP_NOTHING,	" BIT	0,C",
  OP_NOTHING,	" BIT	0,D",
  OP_NOTHING,	" BIT	0,E",
  OP_NOTHING,	" BIT	0,H",
  OP_NOTHING,	" BIT	0,L",
  OP_NOTHING,	" BIT	0,(HL)",
  OP_NOTHING,	" BIT	0,A",

  OP_NOTHING,	" BIT	1,B",		/* CB 48 */
  OP_NOTHING,	" BIT	1,C",
  OP_NOTHING,	" BIT	1,D",
  OP_NOTHING,	" BIT	1,E",
  OP_NOTHING,	" BIT	1,H",
  OP_NOTHING,	" BIT	1,L",
  OP_NOTHING,	" BIT	1,(HL)",
  OP_NOTHING,	" BIT	1,A",

  OP_NOTHING,	" BIT	2,B",		/* CB 50 */
  OP_NOTHING,	" BIT	2,C",
  OP_NOTHING,	" BIT	2,D",
  OP_NOTHING,	" BIT	2,E",
  OP_NOTHING,	" BIT	2,H",
  OP_NOTHING,	" BIT	2,L",
  OP_NOTHING,	" BIT	2,(HL)",
  OP_NOTHING,	" BIT	2,A",

  OP_NOTHING,	" BIT	3,B",		/* CB 58 */
  OP_NOTHING,	" BIT	3,C",
  OP_NOTHING,	" BIT	3,D",
  OP_NOTHING,	" BIT	3,E",
  OP_NOTHING,	" BIT	3,H",
  OP_NOTHING,	" BIT	3,L",
  OP_NOTHING,	" BIT	3,(HL)",
  OP_NOTHING,	" BIT	3,A",

  OP_NOTHING,	" BIT	4,B",		/* CB 60 */
  OP_NOTHING,	" BIT	4,C",
  OP_NOTHING,	" BIT	4,D",
  OP_NOTHING,	" BIT	4,E",
  OP_NOTHING,	" BIT	4,H",
  OP_NOTHING,	" BIT	4,L",
  OP_NOTHING,	" BIT	4,(HL)",
  OP_NOTHING,	" BIT	4,A",

  OP_NOTHING,	" BIT	5,B",		/* CB 68 */
  OP_NOTHING,	" BIT	5,C",
  OP_NOTHING,	" BIT	5,D",
  OP_NOTHING,	" BIT	5,E",
  OP_NOTHING,	" BIT	5,H",
  OP_NOTHING,	" BIT	5,L",
  OP_NOTHING,	" BIT	5,(HL)",
  OP_NOTHING,	" BIT	5,A",

  OP_NOTHING,	" BIT	6,B",		/* CB 70 */
  OP_NOTHING,	" BIT	6,C",
  OP_NOTHING,	" BIT	6,D",
  OP_NOTHING,	" BIT	6,E",
  OP_NOTHING,	" BIT	6,H",
  OP_NOTHING,	" BIT	6,L",
  OP_NOTHING,	" BIT	6,(HL)",
  OP_NOTHING,	" BIT	6,A",

  OP_NOTHING,	" BIT	7,B",		/* CB 78 */
  OP_NOTHING,	" BIT	7,C",
  OP_NOTHING,	" BIT	7,D",
  OP_NOTHING,	" BIT	7,E",
  OP_NOTHING,	" BIT	7,H",
  OP_NOTHING,	" BIT	7,L",
  OP_NOTHING,	" BIT	7,(HL)",
  OP_NOTHING,	" BIT	7,A",

  OP_NOTHING,	" RES	0,B",		/* CB 80 */
  OP_NOTHING,	" RES	0,C",
  OP_NOTHING,	" RES	0,D",
  OP_NOTHING,	" RES	0,E",
  OP_NOTHING,	" RES	0,H",
  OP_NOTHING,	" RES	0,L",
  OP_NOTHING,	" RES	0,(HL)",
  OP_NOTHING,	" RES	0,A",

  OP_NOTHING,	" RES	1,B",		/* CB 88 */
  OP_NOTHING,	" RES	1,C",
  OP_NOTHING,	" RES	1,D",
  OP_NOTHING,	" RES	1,E",
  OP_NOTHING,	" RES	1,H",
  OP_NOTHING,	" RES	1,L",
  OP_NOTHING,	" RES	1,(HL)",
  OP_NOTHING,	" RES	1,A",

  OP_NOTHING,	" RES	2,B",		/* CB 90 */
  OP_NOTHING,	" RES	2,C",
  OP_NOTHING,	" RES	2,D",
  OP_NOTHING,	" RES	2,E",
  OP_NOTHING,	" RES	2,H",
  OP_NOTHING,	" RES	2,L",
  OP_NOTHING,	" RES	2,(HL)",
  OP_NOTHING,	" RES	2,A",

  OP_NOTHING,	" RES	3,B",		/* CB 98 */
  OP_NOTHING,	" RES	3,C",
  OP_NOTHING,	" RES	3,D",
  OP_NOTHING,	" RES	3,E",
  OP_NOTHING,	" RES	3,H",
  OP_NOTHING,	" RES	3,L",
  OP_NOTHING,	" RES	3,(HL)",
  OP_NOTHING,	" RES	3,A",

  OP_NOTHING,	" RES	4,B",		/* CB A0 */
  OP_NOTHING,	" RES	4,C",
  OP_NOTHING,	" RES	4,D",
  OP_NOTHING,	" RES	4,E",
  OP_NOTHING,	" RES	4,H",
  OP_NOTHING,	" RES	4,L",
  OP_NOTHING,	" RES	4,(HL)",
  OP_NOTHING,	" RES	4,A",

  OP_NOTHING,	" RES	5,B",		/* CB A8 */
  OP_NOTHING,	" RES	5,C",
  OP_NOTHING,	" RES	5,D",
  OP_NOTHING,	" RES	5,E",
  OP_NOTHING,	" RES	5,H",
  OP_NOTHING,	" RES	5,L",
  OP_NOTHING,	" RES	5,(HL)",
  OP_NOTHING,	" RES	5,A",

  OP_NOTHING,	" RES	6,B",		/* CB B0 */
  OP_NOTHING,	" RES	6,C",
  OP_NOTHING,	" RES	6,D",
  OP_NOTHING,	" RES	6,E",
  OP_NOTHING,	" RES	6,H",
  OP_NOTHING,	" RES	6,L",
  OP_NOTHING,	" RES	6,(HL)",
  OP_NOTHING,	" RES	6,A",

  OP_NOTHING,	" RES	7,B",		/* CB B8 */
  OP_NOTHING,	" RES	7,C",
  OP_NOTHING,	" RES	7,D",
  OP_NOTHING,	" RES	7,E",
  OP_NOTHING,	" RES	7,H",
  OP_NOTHING,	" RES	7,L",
  OP_NOTHING,	" RES	7,(HL)",
  OP_NOTHING,	" RES	7,A",

  OP_NOTHING,	" SET	0,B",		/* CB C0 */
  OP_NOTHING,	" SET	0,C",
  OP_NOTHING,	" SET	0,D",
  OP_NOTHING,	" SET	0,E",
  OP_NOTHING,	" SET	0,H",
  OP_NOTHING,	" SET	0,L",
  OP_NOTHING,	" SET	0,(HL)",
  OP_NOTHING,	" SET	0,A",

  OP_NOTHING,	" SET	1,B",		/* CB C8 */
  OP_NOTHING,	" SET	1,C",
  OP_NOTHING,	" SET	1,D",
  OP_NOTHING,	" SET	1,E",
  OP_NOTHING,	" SET	1,H",
  OP_NOTHING,	" SET	1,L",
  OP_NOTHING,	" SET	1,(HL)",
  OP_NOTHING,	" SET	1,A",

  OP_NOTHING,	" SET	2,B",		/* CB D0 */
  OP_NOTHING,	" SET	2,C",
  OP_NOTHING,	" SET	2,D",
  OP_NOTHING,	" SET	2,E",
  OP_NOTHING,	" SET	2,H",
  OP_NOTHING,	" SET	2,L",
  OP_NOTHING,	" SET	2,(HL)",
  OP_NOTHING,	" SET	2,A",

  OP_NOTHING,	" SET	3,B",		/* CB D8 */
  OP_NOTHING,	" SET	3,C",
  OP_NOTHING,	" SET	3,D",
  OP_NOTHING,	" SET	3,E",
  OP_NOTHING,	" SET	3,H",
  OP_NOTHING,	" SET	3,L",
  OP_NOTHING,	" SET	3,(HL)",
  OP_NOTHING,	" SET	3,A",

  OP_NOTHING,	" SET	4,B",		/* CB E0 */
  OP_NOTHING,	" SET	4,C",
  OP_NOTHING,	" SET	4,D",
  OP_NOTHING,	" SET	4,E",
  OP_NOTHING,	" SET	4,H",
  OP_NOTHING,	" SET	4,L",
  OP_NOTHING,	" SET	4,(HL)",
  OP_NOTHING,	" SET	4,A",

  OP_NOTHING,	" SET	5,B",		/* CB E8 */
  OP_NOTHING,	" SET	5,C",
  OP_NOTHING,	" SET	5,D",
  OP_NOTHING,	" SET	5,E",
  OP_NOTHING,	" SET	5,H",
  OP_NOTHING,	" SET	5,L",
  OP_NOTHING,	" SET	5,(HL)",
  OP_NOTHING,	" SET	5,A",

  OP_NOTHING,	" SET	6,B",		/* CB F0 */
  OP_NOTHING,	" SET	6,C",
  OP_NOTHING,	" SET	6,D",
  OP_NOTHING,	" SET	6,E",
  OP_NOTHING,	" SET	6,H",
  OP_NOTHING,	" SET	6,L",
  OP_NOTHING,	" SET	6,(HL)",
  OP_NOTHING,	" SET	6,A",

  OP_NOTHING,	" SET	7,B",		/* CB F8 */
  OP_NOTHING,	" SET	7,C",
  OP_NOTHING,	" SET	7,D",
  OP_NOTHING,	" SET	7,E",
  OP_NOTHING,	" SET	7,H",
  OP_NOTHING,	" SET	7,L",
  OP_NOTHING,	" SET	7,(HL)",
  OP_NOTHING,	" SET	7,A"
};

Mnemonics	Instruction_ED[256]=
{
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 00 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 08 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 10 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 18 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 20 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 28 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 30 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 38 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_NOTHING,	" IN	B,(C)",		/* ED 40 */
  OP_NOTHING,	" OUT	(C),B",
  OP_NOTHING,	" SBC	HL,BC",
  OP_MEMORY,	" LD	(0%04XH),BC",
  OP_NOTHING,	" NEG",
  OP_NOTHING,	" RETN",
  OP_NOTHING,	" IM	0",
  OP_NOTHING,	" LD	I,A",

  OP_NOTHING,	" IN	C,(C)",		/* ED 48 */
  OP_NOTHING,	" OUT	(C),C",
  OP_NOTHING,	" ADC	HL,BC",
  OP_MEMORY,	" LD	BC,(0%04XH)",
  OP_UNEXIST,	"+NEG",
  OP_NOTHING,	" RETI",
  OP_UNEXIST,	"+IM	0",
  OP_NOTHING,	" LD	R,A",

  OP_NOTHING,	" IN	D,(C)",		/* ED 50 */
  OP_NOTHING,	" OUT	(C),D",
  OP_NOTHING,	" SBC	HL,DE",
  OP_MEMORY,	" LD	(0%04XH),DE",
  OP_UNEXIST,	"+NEG",
  OP_UNEXIST,	"+RETN",
  OP_NOTHING,	" IM	1",
  OP_NOTHING,	" LD	A,I",

  OP_NOTHING,	" IN	E,(C)",		/* ED 58 */
  OP_NOTHING,	" OUT	(C),E",
  OP_NOTHING,	" ADC	HL,DE",
  OP_MEMORY,	" LD	DE,(0%04XH)",
  OP_UNEXIST,	"+NEG",
  OP_UNEXIST,	"+RETI",
  OP_NOTHING,	" IM	2",
  OP_NOTHING,	" LD	A,R",

  OP_NOTHING,	" IN	H,(C)",		/* ED 60 */
  OP_NOTHING,	" OUT	(C),H",
  OP_NOTHING,	" SBC	HL,HL",
  OP_MEMORY,	"+LD	(0%04XH),HL",
  OP_UNEXIST,	"+NEG",
  OP_UNEXIST,	"+RETN",
  OP_UNEXIST,	"+IM	0",
  OP_NOTHING,	" RRD",

  OP_NOTHING,	" IN	L,(C)",		/* ED 68 */
  OP_NOTHING,	" OUT	(C),L",
  OP_NOTHING,	" ADC	HL,HL",
  OP_MEMORY,	"+LD	HL,(0%04XH)",
  OP_UNEXIST,	"+NEG",
  OP_UNEXIST,	"+RETI",
  OP_UNEXIST,	"+IM	0",
  OP_NOTHING,	" RLD",

  OP_NOTHING,	"+IN	F,(C)",		/* ED 70 */
  OP_UNEXIST,	"+OUT	(C),F",
  OP_NOTHING,	" SBC	HL,SP",
  OP_MEMORY,	" LD	(0%04XH),SP",
  OP_UNEXIST,	"+NEG",
  OP_UNEXIST,	"+RETN",
  OP_UNEXIST,	"+IM	1",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_NOTHING,	" IN	A,(C)",		/* ED 78 */
  OP_NOTHING,	" OUT	(C),A",
  OP_NOTHING,	" ADC	HL,SP",
  OP_MEMORY,	" LD	SP,(0%04XH)",
  OP_UNEXIST,	"+NEG",
  OP_UNEXIST,	"+RETI",
  OP_UNEXIST,	"+IM	2",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 80 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 88 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 90 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED 98 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_NOTHING,	" LDI",		/* ED A0 */
  OP_NOTHING,	" CPI",
  OP_NOTHING,	" INI",
  OP_NOTHING,	" OUTI",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_NOTHING,	" LDD",		/* ED A8 */
  OP_NOTHING,	" CPD",
  OP_NOTHING,	" IND",
  OP_NOTHING,	" OUTD",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_NOTHING,	" LDIR",		/* ED B0 */
  OP_NOTHING,	" CPIR",
  OP_NOTHING,	" INIR",
  OP_NOTHING,	" OTIR",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_NOTHING,	" LDDR",		/* ED B8 */
  OP_NOTHING,	" CPDR",
  OP_NOTHING,	" INDR",
  OP_NOTHING,	" OTDR",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED C0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED C8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED D0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED D8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED E0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED E8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED F0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH",	/* ED F8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH",
};

Mnemonics	Instruction_DD[256]=
{
  OP_SKIP,	"+DB	0%02XH",	/* DD 00 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 08 */
  OP_NOTHING,	" ADD	IX,BC",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 10 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 18 */
  OP_NOTHING,	" ADD	IX,DE",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 20 */
  OP_NUM_16,	" LD	IX,0%04XH",
  OP_MEMORY,	" LD	(0%04XH),IX",
  OP_NOTHING,	" INC	IX",
  OP_NOTHING,	"+INC	IXH",
  OP_NOTHING,	"+DEC	IXH",
  OP_NUM_8,	"+LD	IXH,0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 28 */
  OP_NOTHING,	" ADD	IX,IX",
  OP_MEMORY,	" LD	IX,(0%04XH)",
  OP_NOTHING,	" DEC	IX",
  OP_NOTHING,	"+INC	IXL",
  OP_NOTHING,	"+DEC	IXL",
  OP_NUM_8,	"+LD	IXL,0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 30 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_INDEX,	" INC	(IX%+d)",
  OP_INDEX,	" DEC	(IX%+d)",
  OP_IDX_NUM,	" LD	(IX%+d),0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 38 */
  OP_NOTHING,	" ADD	IX,SP",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 40 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	B,IXH",
  OP_NOTHING,	"+LD	B,IXL",
  OP_INDEX,	" LD	B,(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 48 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	C,IXH",
  OP_NOTHING,	"+LD	C,IXL",
  OP_INDEX,	" LD	C,(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 50 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	D,IXH",
  OP_NOTHING,	"+LD	D,IXL",
  OP_INDEX,	" LD	D,(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 58 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	E,IXH",
  OP_NOTHING,	"+LD	E,IXL",
  OP_INDEX,	" LD	E,(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_NOTHING,	"+LD	IXH,B",		/* DD 60 */
  OP_NOTHING,	"+LD	IXH,C",
  OP_NOTHING,	"+LD	IXH,D",
  OP_NOTHING,	"+LD	IXH,E",
  OP_NOTHING,	"+LD	IXH,IXH",
  OP_NOTHING,	"+LD	IXH,IXL",
  OP_INDEX,	" LD	H,(IX%+d)",
  OP_NOTHING,	"+LD	IXH,A",

  OP_NOTHING,	"+LD	IXL,B",		/* DD 68 */
  OP_NOTHING,	"+LD	IXL,C",
  OP_NOTHING,	"+LD	IXL,D",
  OP_NOTHING,	"+LD	IXL,E",
  OP_NOTHING,	"+LD	IXL,IXH",
  OP_NOTHING,	"+LD	IXL,IXL",
  OP_INDEX,	" LD	L,(IX%+d)",
  OP_NOTHING,	"+LD	IXL,A",

  OP_INDEX,	" LD	(IX%+d),B",	/* DD 70 */
  OP_INDEX,	" LD	(IX%+d),C",
  OP_INDEX,	" LD	(IX%+d),D",
  OP_INDEX,	" LD	(IX%+d),E",
  OP_INDEX,	" LD	(IX%+d),H",
  OP_INDEX,	" LD	(IX%+d),L",
  OP_SKIP,	"+DB	0%02XH",
  OP_INDEX,	" LD	(IX%+d),A",

  OP_SKIP,	"+DB	0%02XH",	/* DD 78 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	A,IXH",
  OP_NOTHING,	"+LD	A,IXL",
  OP_INDEX,	" LD	A,(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 80 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+ADD	A,IXH",
  OP_NOTHING,	"+ADD	A,IXL",
  OP_INDEX,	" ADD	A,(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 88 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+ADC	A,IXH",
  OP_NOTHING,	"+ADC	A,IXL",
  OP_INDEX,	" ADC	A,(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 90 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+SUB	IXH",
  OP_NOTHING,	"+SUB	IXL",
  OP_INDEX,	" SUB	(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD 98 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+SBC	A,IXH",
  OP_NOTHING,	"+SBC	A,IXL",
  OP_INDEX,	" SBC	A,(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD A0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+AND	IXH",
  OP_NOTHING,	"+AND	IXL",
  OP_INDEX,	" AND	(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD A8 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+XOR	IXH",
  OP_NOTHING,	"+XOR	IXL",
  OP_INDEX,	" XOR	(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD B0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+OR	IXH",
  OP_NOTHING,	"+OR	IXL",
  OP_INDEX,	" OR	(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD B8 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+CP	IXH",
  OP_NOTHING,	"+CP	IXL",
  OP_INDEX,	" CP	(IX%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD C0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD C8 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_PREFIX,	NULL,
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD D0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD D8 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD E0 */
  OP_NOTHING,	" POP	IX",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	" EX	IX,(SP)",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	" PUSH	IX",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD E8 */
  OP_NOTHING,	" JP	(IX)",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD F0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* DD F8 */
  OP_NOTHING,	" LD	SP,IX",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
};

Mnemonics	Instruction_DD_CB[256]=
{
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 00 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RLC	(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 08 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RRC	(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 10 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RL	(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 18 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RR	(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 20 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SLA	(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 28 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SRA	(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 30 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	"+SLL	(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 38 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SRL	(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_INDEX,	"+BIT	0,(IX%+d)",			/* DD CB d 40 */
  OP_INDEX,	"+BIT	0,(IX%+d)",
  OP_INDEX,	"+BIT	0,(IX%+d)",
  OP_INDEX,	"+BIT	0,(IX%+d)",
  OP_INDEX,	"+BIT	0,(IX%+d)",
  OP_INDEX,	"+BIT	0,(IX%+d)",
  OP_INDEX,	" BIT	0,(IX%+d)",
  OP_INDEX,	"+BIT	0,(IX%+d)",

  OP_INDEX,	"+BIT	1,(IX%+d)",			/* DD CB d 48 */
  OP_INDEX,	"+BIT	1,(IX%+d)",
  OP_INDEX,	"+BIT	1,(IX%+d)",
  OP_INDEX,	"+BIT	1,(IX%+d)",
  OP_INDEX,	"+BIT	1,(IX%+d)",
  OP_INDEX,	"+BIT	1,(IX%+d)",
  OP_INDEX,	" BIT	1,(IX%+d)",
  OP_INDEX,	"+BIT	1,(IX%+d)",

  OP_INDEX,	"+BIT	2,(IX%+d)",			/* DD CB d 50 */
  OP_INDEX,	"+BIT	2,(IX%+d)",
  OP_INDEX,	"+BIT	2,(IX%+d)",
  OP_INDEX,	"+BIT	2,(IX%+d)",
  OP_INDEX,	"+BIT	2,(IX%+d)",
  OP_INDEX,	"+BIT	2,(IX%+d)",
  OP_INDEX,	" BIT	2,(IX%+d)",
  OP_INDEX,	"+BIT	2,(IX%+d)",

  OP_INDEX,	"+BIT	3,(IX%+d)",			/* DD CB d 58 */
  OP_INDEX,	"+BIT	3,(IX%+d)",
  OP_INDEX,	"+BIT	3,(IX%+d)",
  OP_INDEX,	"+BIT	3,(IX%+d)",
  OP_INDEX,	"+BIT	3,(IX%+d)",
  OP_INDEX,	"+BIT	3,(IX%+d)",
  OP_INDEX,	" BIT	3,(IX%+d)",
  OP_INDEX,	"+BIT	3,(IX%+d)",

  OP_INDEX,	"+BIT	4,(IX%+d)",			/* DD CB d 60 */
  OP_INDEX,	"+BIT	4,(IX%+d)",
  OP_INDEX,	"+BIT	4,(IX%+d)",
  OP_INDEX,	"+BIT	4,(IX%+d)",
  OP_INDEX,	"+BIT	4,(IX%+d)",
  OP_INDEX,	"+BIT	4,(IX%+d)",
  OP_INDEX,	" BIT	4,(IX%+d)",
  OP_INDEX,	"+BIT	4,(IX%+d)",

  OP_INDEX,	"+BIT	5,(IX%+d)",			/* DD CB d 68 */
  OP_INDEX,	"+BIT	5,(IX%+d)",
  OP_INDEX,	"+BIT	5,(IX%+d)",
  OP_INDEX,	"+BIT	5,(IX%+d)",
  OP_INDEX,	"+BIT	5,(IX%+d)",
  OP_INDEX,	"+BIT	5,(IX%+d)",
  OP_INDEX,	" BIT	5,(IX%+d)",
  OP_INDEX,	"+BIT	5,(IX%+d)",

  OP_INDEX,	"+BIT	6,(IX%+d)",			/* DD CB d 70 */
  OP_INDEX,	"+BIT	6,(IX%+d)",
  OP_INDEX,	"+BIT	6,(IX%+d)",
  OP_INDEX,	"+BIT	6,(IX%+d)",
  OP_INDEX,	"+BIT	6,(IX%+d)",
  OP_INDEX,	"+BIT	6,(IX%+d)",
  OP_INDEX,	" BIT	6,(IX%+d)",
  OP_INDEX,	"+BIT	6,(IX%+d)",

  OP_INDEX,	"+BIT	7,(IX%+d)",			/* DD CB d 78 */
  OP_INDEX,	"+BIT	7,(IX%+d)",
  OP_INDEX,	"+BIT	7,(IX%+d)",
  OP_INDEX,	"+BIT	7,(IX%+d)",
  OP_INDEX,	"+BIT	7,(IX%+d)",
  OP_INDEX,	"+BIT	7,(IX%+d)",
  OP_INDEX,	" BIT	7,(IX%+d)",
  OP_INDEX,	"+BIT	7,(IX%+d)",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 80 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	0,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 88 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	1,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 90 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	2,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d 98 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	3,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d A0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	4,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d A8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	5,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d B0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	6,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d B8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	7,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d C0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	0,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d C8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	1,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d D0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	2,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d D8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	3,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d E0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	4,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d E8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	5,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* DD CB d F0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	6,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/*DD CB d F8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	7,(IX%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
};







Mnemonics	Instruction_FD[256]=
{
  OP_SKIP,	"+DB	0%02XH",	/* FD 00 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 08 */
  OP_NOTHING,	" ADD	IY,BC",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 10 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 18 */
  OP_NOTHING,	" ADD	IY,DE",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 20 */
  OP_NUM_16,	" LD	IY,0%04XH",
  OP_MEMORY,	" LD	(0%04XH),IY",
  OP_NOTHING,	" INC	IY",
  OP_NOTHING,	"+INC	IYH",
  OP_NOTHING,	"+DEC	IYH",
  OP_NUM_8,	"+LD	IYH,0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 28 */
  OP_NOTHING,	" ADD	IY,IY",
  OP_MEMORY,	" LD	IY,(0%04XH)",
  OP_NOTHING,	" DEC	IY",
  OP_NOTHING,	"+INC	IYL",
  OP_NOTHING,	"+DEC	IYL",
  OP_NUM_8,	"+LD	IYL,0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 30 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_INDEX,	" INC	(IY%+d)",
  OP_INDEX,	" DEC	(IY%+d)",
  OP_IDX_NUM,	" LD	(IY%+d),0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 38 */
  OP_NOTHING,	" ADD	IY,SP",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 40 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	B,IYH",
  OP_NOTHING,	"+LD	B,IYL",
  OP_INDEX,	" LD	B,(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 48 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	C,IYH",
  OP_NOTHING,	"+LD	C,IYL",
  OP_INDEX,	" LD	C,(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 50 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	D,IYH",
  OP_NOTHING,	"+LD	D,IYL",
  OP_INDEX,	" LD	D,(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 58 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	E,IYH",
  OP_NOTHING,	"+LD	E,IYL",
  OP_INDEX,	" LD	E,(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_NOTHING,	"+LD	IYH,B",		/* FD 60 */
  OP_NOTHING,	"+LD	IYH,C",
  OP_NOTHING,	"+LD	IYH,D",
  OP_NOTHING,	"+LD	IYH,E",
  OP_NOTHING,	"+LD	IYH,IYH",
  OP_NOTHING,	"+LD	IYH,IYL",
  OP_INDEX,	" LD	H,(IY%+d)",
  OP_NOTHING,	"+LD	IYH,A",

  OP_NOTHING,	"+LD	IYL,B",		/* FD 68 */
  OP_NOTHING,	"+LD	IYL,C",
  OP_NOTHING,	"+LD	IYL,D",
  OP_NOTHING,	"+LD	IYL,E",
  OP_NOTHING,	"+LD	IYL,IYH",
  OP_NOTHING,	"+LD	IYL,IYL",
  OP_INDEX,	" LD	L,(IY%+d)",
  OP_NOTHING,	"+LD	IYL,A",

  OP_INDEX,	" LD	(IY%+d),B",	/* FD 70 */
  OP_INDEX,	" LD	(IY%+d),C",
  OP_INDEX,	" LD	(IY%+d),D",
  OP_INDEX,	" LD	(IY%+d),E",
  OP_INDEX,	" LD	(IY%+d),H",
  OP_INDEX,	" LD	(IY%+d),L",
  OP_SKIP,	"+DB	0%02XH",
  OP_INDEX,	" LD	(IY%+d),A",

  OP_SKIP,	"+DB	0%02XH",	/* FD 78 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+LD	A,IYH",
  OP_NOTHING,	"+LD	A,IYL",
  OP_INDEX,	" LD	A,(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 80 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+ADD	A,IYH",
  OP_NOTHING,	"+ADD	A,IYL",
  OP_INDEX,	" ADD	A,(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 88 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+ADC	A,IYH",
  OP_NOTHING,	"+ADC	A,IYL",
  OP_INDEX,	" ADC	A,(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 90 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+SUB	IYH",
  OP_NOTHING,	"+SUB	IYL",
  OP_INDEX,	" SUB	(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD 98 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+SBC	A,IYH",
  OP_NOTHING,	"+SBC	A,IYL",
  OP_INDEX,	" SBC	A,(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD A0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+AND	IYH",
  OP_NOTHING,	"+AND	IYL",
  OP_INDEX,	" AND	(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD A8 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+XOR	IYH",
  OP_NOTHING,	"+XOR	IYL",
  OP_INDEX,	" XOR	(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD B0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+OR	IYH",
  OP_NOTHING,	"+OR	IYL",
  OP_INDEX,	" OR	(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD B8 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	"+CP	IYH",
  OP_NOTHING,	"+CP	IYL",
  OP_INDEX,	" CP	(IY%+d)",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD C0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD C8 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_PREFIX,	NULL,
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD D0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD D8 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD E0 */
  OP_NOTHING,	" POP	IY",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	" EX	IY,(SP)",
  OP_SKIP,	"+DB	0%02XH",
  OP_NOTHING,	" PUSH	IY",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD E8 */
  OP_NOTHING,	" JP	(IY)",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD F0 */
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",

  OP_SKIP,	"+DB	0%02XH",	/* FD F8 */
  OP_NOTHING,	" LD	SP,IY",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
  OP_SKIP,	"+DB	0%02XH",
};

Mnemonics	Instruction_FD_CB[256]=
{
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 00 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RLC	(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 08 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RRC	(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 10 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RL	(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 18 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RR	(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 20 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SLA	(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 28 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SRA	(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 30 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	"+SLL	(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 38 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SRL	(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_INDEX,	"+BIT	0,(IY%+d)",			/* FD CB d 40 */
  OP_INDEX,	"+BIT	0,(IY%+d)",
  OP_INDEX,	"+BIT	0,(IY%+d)",
  OP_INDEX,	"+BIT	0,(IY%+d)",
  OP_INDEX,	"+BIT	0,(IY%+d)",
  OP_INDEX,	"+BIT	0,(IY%+d)",
  OP_INDEX,	" BIT	0,(IY%+d)",
  OP_INDEX,	"+BIT	0,(IY%+d)",

  OP_INDEX,	"+BIT	1,(IY%+d)",			/* FD CB d 48 */
  OP_INDEX,	"+BIT	1,(IY%+d)",
  OP_INDEX,	"+BIT	1,(IY%+d)",
  OP_INDEX,	"+BIT	1,(IY%+d)",
  OP_INDEX,	"+BIT	1,(IY%+d)",
  OP_INDEX,	"+BIT	1,(IY%+d)",
  OP_INDEX,	" BIT	1,(IY%+d)",
  OP_INDEX,	"+BIT	1,(IY%+d)",

  OP_INDEX,	"+BIT	2,(IY%+d)",			/* FD CB d 50 */
  OP_INDEX,	"+BIT	2,(IY%+d)",
  OP_INDEX,	"+BIT	2,(IY%+d)",
  OP_INDEX,	"+BIT	2,(IY%+d)",
  OP_INDEX,	"+BIT	2,(IY%+d)",
  OP_INDEX,	"+BIT	2,(IY%+d)",
  OP_INDEX,	" BIT	2,(IY%+d)",
  OP_INDEX,	"+BIT	2,(IY%+d)",

  OP_INDEX,	"+BIT	3,(IY%+d)",			/* FD CB d 58 */
  OP_INDEX,	"+BIT	3,(IY%+d)",
  OP_INDEX,	"+BIT	3,(IY%+d)",
  OP_INDEX,	"+BIT	3,(IY%+d)",
  OP_INDEX,	"+BIT	3,(IY%+d)",
  OP_INDEX,	"+BIT	3,(IY%+d)",
  OP_INDEX,	" BIT	3,(IY%+d)",
  OP_INDEX,	"+BIT	3,(IY%+d)",

  OP_INDEX,	"+BIT	4,(IY%+d)",			/* FD CB d 60 */
  OP_INDEX,	"+BIT	4,(IY%+d)",
  OP_INDEX,	"+BIT	4,(IY%+d)",
  OP_INDEX,	"+BIT	4,(IY%+d)",
  OP_INDEX,	"+BIT	4,(IY%+d)",
  OP_INDEX,	"+BIT	4,(IY%+d)",
  OP_INDEX,	" BIT	4,(IY%+d)",
  OP_INDEX,	"+BIT	4,(IY%+d)",

  OP_INDEX,	"+BIT	5,(IY%+d)",			/* FD CB d 68 */
  OP_INDEX,	"+BIT	5,(IY%+d)",
  OP_INDEX,	"+BIT	5,(IY%+d)",
  OP_INDEX,	"+BIT	5,(IY%+d)",
  OP_INDEX,	"+BIT	5,(IY%+d)",
  OP_INDEX,	"+BIT	5,(IY%+d)",
  OP_INDEX,	" BIT	5,(IY%+d)",
  OP_INDEX,	"+BIT	5,(IY%+d)",

  OP_INDEX,	"+BIT	6,(IY%+d)",			/* FD CB d 70 */
  OP_INDEX,	"+BIT	6,(IY%+d)",
  OP_INDEX,	"+BIT	6,(IY%+d)",
  OP_INDEX,	"+BIT	6,(IY%+d)",
  OP_INDEX,	"+BIT	6,(IY%+d)",
  OP_INDEX,	"+BIT	6,(IY%+d)",
  OP_INDEX,	" BIT	6,(IY%+d)",
  OP_INDEX,	"+BIT	6,(IY%+d)",

  OP_INDEX,	"+BIT	7,(IY%+d)",			/* FD CB d 78 */
  OP_INDEX,	"+BIT	7,(IY%+d)",
  OP_INDEX,	"+BIT	7,(IY%+d)",
  OP_INDEX,	"+BIT	7,(IY%+d)",
  OP_INDEX,	"+BIT	7,(IY%+d)",
  OP_INDEX,	"+BIT	7,(IY%+d)",
  OP_INDEX,	" BIT	7,(IY%+d)",
  OP_INDEX,	"+BIT	7,(IY%+d)",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 80 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	0,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 88 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	1,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 90 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	2,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d 98 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	3,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d A0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	4,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d A8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	5,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d B0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	6,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d B8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" RES	7,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d C0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	0,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d C8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	1,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d D0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	2,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d D8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	3,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d E0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	4,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d E8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	5,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d F0 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	6,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",

  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",	/* FD CB d F8 */
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
  OP_INDEX,	" SET	7,(IY%+d)",
  OP_UNEXIST,	"+DB	0%02XH,0%02XH,0%02XH,0%02XH",
};






void	disasm( void );
void	make_label( void );
int	get_option( FILE * );

long		disasm_start;		/* ե֥볫ϥɥ쥹 */
long		disasm_size;		/* ե֥ץॵ */

int		option_line;		/* եθ߲ϹԿ */
int		verbose;		/* ٤ƤξϤ */
int		tab_width;		/* TAB  (8ܿ) */

unsigned char	prog[0x10000];		/* ץ */

unsigned char	attr[0x10000];		/* ȥӥ塼 */
unsigned short	label[0x10000];		/* ٥         */
unsigned char	label_buf[32768];	/* ̾ե٥   */
unsigned int	label_ptr;		/* ̾ե٥Υݥ */

/*
   attr[] 
     0x00		ڥ			 ͤǽ
     0x20 + 031	DB̿(1 32Х)
     0x40 + 031	DC̿(1 32Х)
     0x60 + 031	DW̿(2 62Х)
     0x80 + 0127	DS̿(1128Х)
*/
/*
   label[] 
     0x0000		٥ʤ			 ͤǽ
     0x4000 + 016383	ư٥Υ٥ֹ
     0x8000 + 032767	桼٥Υ٥̾ХåեΥݥ
*/

enum{
  ATTR_NO = 0,
  ATTR_DB = 0x20,
  ATTR_DC = 0x40,
  ATTR_DW = 0x60,
  ATTR_DS = 0x80,
};
enum{
  LABEL_NO   = 0,
  LABEL_AUTO = 0x4000,
  LABEL_USER = 0x8000,
};


void	help( void )
{
  fprintf( stderr, "Tiny Z80 Disassembler Ver 0.1 by fukuchan\n" );
  fprintf( stderr, "Usage: dasm88 [-org n [-size n] ] [-nv] [-t n] [-f option_file] binary_file\n" );
  fprintf( stderr, "    -org n          - Set Disassemble start address n\n");
  fprintf( stderr, "    -size n         - Set Disassemble size n\n");
  fprintf( stderr, "    -f option_file  - Specity Option-File\n");
  fprintf( stderr, "    -nv             - Not verbose output\n" );
  fprintf( stderr, "    -t n            - Set tab width n ( align 8*x )\n");
  fprintf( stderr, "    binary_file     - Binary-File\n");
  exit(1);
}

int	main( int argc, char *argv[] )
{
  FILE	*fp, *fpx;
  int	i, c, size;
  char	*binary_name = NULL;
  char	*config_name = NULL;

  disasm_start = -1;
  disasm_size  = -1;
  verbose      = TRUE;
  tab_width    = 8;


  if( argc==1 ) help();

  for( i=1; i<argc; i++ ){
    if      ( strcmp( argv[i], "-help" )==0 ){			/* -help */
      help();

    }else if( strcmp( argv[i], "-org" )==0 ){			/* -org */
      if( ++i>=argc ){
	fprintf(stderr,"Bad argument ( -org ).\n");
	exit(1);
      }
      disasm_start = strtol(argv[i],(char**)NULL,0) & 0xffff;

    }else if( strcmp( argv[i], "-size" )==0 ){			/* -size */
      if( disasm_start<0 || ++i>=argc ){
	fprintf(stderr,"Bad argument ( -size ).\n");
	exit(1);
      }
      disasm_size = strtol(argv[i],(char**)NULL,0) & 0xffff;

    }else if( strcmp( argv[i], "-f" )==0 ){			/* -f */
      if( ++i>=argc ) {
	fprintf(stderr,"Bad argument ( -f ).\n");
	exit(1);
      }
      config_name = argv[i];

    }else if( strcmp( argv[i], "-nv" )==0 ){			/* -nv */
      verbose = FALSE;

    }else if( strcmp( argv[i], "-t" )==0 ){			/* -t */
      if( ++i>=argc ){
	fprintf(stderr,"Bad argument ( -t ).\n");
	exit(1);
      }
      tab_width = ( strtol(argv[i],(char**)NULL,0)+7 ) & 0xf8;

    }else if( *argv[i]=='-' ){					/* - */
      fprintf(stderr,"Unknown option.\n");
      exit(1);

    }else if( binary_name==NULL ){				/* binary_f */
      binary_name = argv[i];

    }else{							/* Another */
      fprintf(stderr,"Bad argument ( dupulicate binary-file ).\n");
      exit(1);
    }
  }

  if( binary_name==NULL ){
    fprintf( stderr, "Bad argument ( binary-file not exist ).\n");
    exit(1);
  }



	/* Хʥ ե륪ץ */

  if( !(fp=fopen(binary_name,"rb")) ){
    fprintf( stderr, "\n%s: Can't open file %s\n", argv[0], binary_name );
    exit(1);
  }

	/* ե륪ץ */

  if( config_name ){
    if( !(fpx=fopen(config_name,"r")) ){
      fprintf( stderr, "\n%s: Can't open file %s\n", argv[0], config_name );
      exit(1);
    }
  }
  option_line = 0;



	/* Хʥ ե֥ */

  while( config_name || binary_name ){

    for(i=0;i<0x10000;i++) prog[i]=attr[i]=label[i]=0;	/*  */
    label_ptr = 0;

    if( config_name ){			/* եˤ */
      if( get_option( fpx )==FALSE ){
	fclose( fpx );
	config_name = NULL;
      }
    }

    if( disasm_start < 0 ) disasm_start = 0;	/* Хʥ˳Ǽ */
    if( disasm_size < 0 ){
      for( size=0; (c=getc(fp))!=EOF; size++ ){
	prog[ (disasm_start + size)&0xffff ] = c&0xff;
      }
      disasm_size = size;
    }else{
      for( size=0; size<disasm_size; size++ ){
	if( (c=getc(fp))==EOF ) break;
	prog[ (disasm_start + size)&0xffff ] = c&0xff;
      }
    }

    if( size==0 ) break;
    if( size>0xffff ){
      fprintf(stderr,"Binary file too big! ( max 65536byte! ).\n");
      exit(1);
    }

    make_label();			/* ٥ơ֥κ */
    disasm();				/* ե֥  ɽ */


    disasm_start = -1;
    disasm_size  = -1;
  }

	/* λ */

  fclose( fp );

  exit(0);
}





/*----------------------------------------------------------------------*/
/* եɤ߹ߡattr[]label[] 򥻥å			*/
/*----------------------------------------------------------------------*/

/*
  buf[] 顢ʸФstr[]롣
  򡢥̵֤롣; ʹߤϥȤȤߤʤ
  ߤʸΰ֤򼨤ݥ󥿤ɬס
  ֤ͤϡʥݥֹ档ʸ̵ϡ0
*/
int	get_next_strings_ptr( char buf[], char str[], int ptr )
{
  int p=0;
  while(1){
    if     ( buf[ptr]==';' || buf[ptr]=='\0' || buf[ptr]=='\n' ) return 0;
    else if( buf[ptr]==' ' || buf[ptr]=='\t' )                   ptr++;
    else                                                         break;
  }
  while(1){
    if     ( buf[ptr]==';' || buf[ptr]=='\0' || buf[ptr]=='\n' ||
	     buf[ptr]==' ' || buf[ptr]=='\t' ){
      str[p]='\0';
      return( ptr );
    }else{
      str[p] = buf[ptr];
      p++, ptr++;
    }
  }
}

/*
  line 顢2ĤʸȤ*start*size ˳Ǽ롣
  եޥåȤϡ16ʿ  XXXX YYYY  ⤷  XXXX #YYYY
  Ԥϡstart end Ԥ start size ̣롣
  ֤ͤϡʸ󤬼ФС
*/
int	get_start_and_size( char line[], int ptr, int *start, int *size )
{
  char buf[1024];

  if( (ptr=get_next_strings_ptr( line, buf, ptr )) != 0 ){
    *start = strtol( buf, NULL, 16 ) & 0xffff;
    if( (ptr=get_next_strings_ptr( line, buf, ptr )) != 0 ){
      if( buf[0]=='#' ) *size = strtol( &buf[1], NULL, 16 ) & 0xffff;
      else              *size =(strtol( buf, NULL, 16 ) -(*start) +1) & 0xffff;
    }else{
      return FALSE;
    }
  }else{
    return FALSE;
  }
  return TRUE;
}

/*
   ե
   ORG ̵ꤷơdisasm_startdisasm_size ϽƤʤƤ
   ʤޤattr[]label[]label_ptr  0 ǽ
   option_line Ϸ³ŪˡեβϺѤ߹ԤؤƤ
   ե뤬ϤǤС֤롣
   եȤʤС֤롣
   ե˰۾郎Сǡ
*/
void	error( char *s )
{
  fprintf( stderr, s );
  fprintf( stderr, " in %d.\n", option_line );
  exit(1);
}
int	get_option( FILE *fpx )
{
  int	i, j, start, size;
  char	line[ OPTION_MAX_CHARS+1 ];
  char	buf[  OPTION_MAX_CHARS+1 ];
  int	line_ptr;
  char *opt_define[]={ "ORG", "END", "DB", "DC", "DW", "DS", "LABEL", NULL };
  enum{                xORG,  xEND,  xDB,  xDC,  xDW,  xDS,  xLABEL  };

  while( fgets( line, OPTION_MAX_CHARS, fpx )!=NULL ){
    option_line++;
    line[ OPTION_MAX_CHARS ]='\0';
    line_ptr = 0;

    if( (line_ptr=get_next_strings_ptr( line, buf, line_ptr )) != 0 ){
      for( i=0; opt_define[i]; i++ )
        if( !strcmp( buf, opt_define[i] ) ) break;


      switch(i){
      case xORG:
	if( get_start_and_size( line, line_ptr, &start, &size ) ){
	  if( start<0 || start>0xffff ) error("ORG: Start is bad");
	  if( size<0  || size>0x10000 ) error("ORG: End or Size is bad");
	  disasm_start = start;
	  disasm_size  = size;
	}else{
	  error("ORG: Bad format");
	}
	break;

      case xEND:
	return TRUE;

      case xDB:
	if( get_start_and_size( line, line_ptr, &start, &size ) ){
	  if( start<0 || start>0xffff ) error("DB: Start is bad");
	  if( size<1  || size>32 )      error("DB: End or Size is bad");
	  attr[ start ] = ATTR_DB | (size-1);
	  for( j=start+1; j<start+size; j++ ){
	    attr[ j&0xffff ] = 0;
	  }
	}else{
	  error("DB: Bad format");
	}
	break;

      case xDC:
	if( get_start_and_size( line, line_ptr, &start, &size ) ){
	  if( start<0 || start>0xffff ) error("DC: Start is bad");
	  if( size<1  || size>32 )      error("DC: End or Size is bad");
	  attr[ start ] = ATTR_DC | (size-1);
	  for( j=start+1; j<start+size; j++ ){
	    attr[ j&0xffff ] = 0;
	  }
	}else{
	  error("DC: Bad format");
	}
	break;

      case xDW:
	if( get_start_and_size( line, line_ptr, &start, &size ) ){
	  if( start<0 || start>0xffff ) error("DW: Start is bad");
	  if( size<2  || size>64 )      error("DW: End or Size is bad");
	  if( (size&1)==1 )             error("DW: Size is odd");
	  attr[ start ] = ATTR_DW | (size/2-1);
	  for( j=start+1; j<start+size; j++ ){
	    attr[ j&0xffff ] = 0;
	  }
	}else{
	  error("DW: Bad format");
	}
	break;

      case xDS:
	if( get_start_and_size( line, line_ptr, &start, &size ) ){
	  if( start<0 || start>0xffff ) error("DS: Start is bad");
	  if( size<1  || size>128 )     error("DS: End or Size is bad");
	  attr[ start ] = ATTR_DS | (size-1);
	  for( j=start+1; j<start+size; j++ ){
	    attr[ j&0xffff ] = 0;
	  }
	}else{
	  error("DS: Bad format");
	}
	break;

      case xLABEL:
	if( (line_ptr=get_next_strings_ptr( line, buf, line_ptr )) != 0 ){
	  start = strtol( buf, NULL, 16 );
	  if( start<0 || start>0xffff ) error("LABEL: Addr is bad");
	  label[ start ] = LABEL_AUTO;
	}else{
	  error("LABEL: Bad format");
	}
	break;

      default:
	size = strlen(buf) +1;
	if( size + label_ptr < 32768 ){
	  strcpy( &label_buf[label_ptr], buf );
	  if( (line_ptr=get_next_strings_ptr( line, buf, line_ptr )) != 0 ){
	    start = strtol( buf, NULL, 16 );
	    if( start<0 || start>0xffff ) error("Users label: Addr is bad");
	    label[ start ] = LABEL_USER | label_ptr;
	    label_ptr += size;
	  }else{
	    error("Users label: Bad format");
	  }
	}else{
	  error("Users label: Out of label buffer");
	}
	break;
      }
    }
  }
  return FALSE;
}




/*------------------------------------------------------*/
/* ٥κ						*/
/*------------------------------------------------------*/
void	make_label( void )
{
  int	addr  = disasm_start;
  int	count = disasm_size;
  int	d, i, num;
  Mnemonics *Inst;

	/* JP,JR,DJNZ,CALL ʤɤ̿ʬ˥٥ޡĤ */

  while( 1 ){

    if( attr[addr] & ATTR_DS ){		/* DS */

      d = (attr[addr]&0x7f) +1;

    }else if( (attr[addr] & 0x60) == ATTR_DB ){

      d = (attr[addr]&0x1f) +1;

    }else if( (attr[addr] & 0x60) == ATTR_DC ){

      d = (attr[addr]&0x1f) +1;

    }else if( (attr[addr] & 0x60) == ATTR_DW ){

      d = ((attr[addr]&0x1f) +1) *2;

    }else{

      Inst = &Instruction[ prog[addr] ];
      switch( Inst->type ){
      case OP_NOTHING:	d=1;		break;
      case OP_NUM_8:	d=2;		break;
      case OP_NUM_16:	d=3;		break;
      case OP_MEMORY:	d=3;		break;

      case OP_ADR_ABS:	num  = prog[(addr+1)&0xffff];
			num += prog[(addr+2)&0xffff]*256;
			num &= 0xffff;
			if( label[ num ]==0 ) label[ num ] = LABEL_AUTO;
			d=3;		break;

      case OP_ADR_REL:	num  = addr +2;
			num += (signed char)prog[(addr+1)&0xffff];
			num &= 0xffff;
			if( label[ num ]==0 ) label[ num ] = LABEL_AUTO;
			d=2;		break;

      case OP_PREFIX:
	switch( prog[addr] ){
	case 0xCB:	Inst = &Instruction_CB[ prog[addr+1] ];	break;
	case 0xDD:	Inst = &Instruction_DD[ prog[addr+1] ];	break;
	case 0xED:	Inst = &Instruction_ED[ prog[addr+1] ];	break;
	case 0xFD:	Inst = &Instruction_FD[ prog[addr+1] ];	break;
	default:	fprintf(stderr,"Internal Error 2!!\n");	exit(1);
	}
	switch( Inst->type ){
	case OP_NOTHING:
	case OP_UNEXIST:	d=2;	break;
	case OP_NUM_8:
	case OP_INDEX:		d=3;	break;
	case OP_NUM_16:	
	case OP_MEMORY:	
	case OP_IDX_NUM:
	case OP_PREFIX:		d=4;	break;
	case OP_SKIP:		d=1;	break;
	}
	break;
      }
    }

    addr  += d;
    count -= d;
    if( count<0 ) break;

  }


	/* ư٥ޡΤĤϤֹ򿶤롣 */

  num=1;
  for( i=0; i<0x10000; i++ ){
    if( (label[i]&0xc000)==LABEL_AUTO ){
      label[i] |= num;
      num ++;
    }
  }
  if( num>16383 ){
    fprintf(stderr,"Opps! Label number is greater than 16383!\n");
  }

}







/*------------------------------------------------------*/
/* ե֥						*/
/*------------------------------------------------------*/
static 	void	printf_head( int addr, int num )
{
  int i, j, size;

	/* ɥ쥹ɽ */

  if( verbose ){
    printf("%04X ",addr);
  }

	/* 󥹥ȥ饯󥳡ɤɽ */

  if( verbose ){

    if( num<0 ){				/* DS ξ㳰 */
      printf("00  x%02XH ",-num);
      printf("  ");
    }else{				/* ̾ϡnumɤĹ򼨤 */
      switch( num ){
      case 1:
	printf("%02X       ",prog[addr]);
	printf("  ");
	break;
      case 2:
	printf("%02X%02X     ",prog[addr],prog[addr+1]);
	if( label[addr+1] ) printf("*");
	else                printf(" ");
	if( attr[addr+1]  ) printf("#");
	else                printf(" ");
	break;
      case 3:
	printf("%02X%02X%02X   ",prog[addr],prog[addr+1],prog[addr+2]);
	if( label[addr+1] || label[addr+2] ) printf("*");
	else                                 printf(" ");
	if( attr[addr+1]  || attr[addr+2]  ) printf("#");
	else                                 printf(" ");
	break;
      case 4:
	printf("%02X%02X%02X%02X ",prog[addr],prog[addr+1],prog[addr+2],prog[addr+3]);
	if( label[addr+1] || label[addr+2] || label[addr+3] ) printf("*");
	else                                                  printf(" ");
	if( attr[addr+1]  || attr[addr+2]  || attr[addr+3]  ) printf("#");
	else                                                  printf(" ");
	break;
      default:
	fprintf(stderr,"Internal Error 3 \n");
	exit(1);
      }
    }
  }


	/* ٥(ڤӡ)ɽ */

  if( label[addr] ) {
    if( (label[addr]&0xc000)==LABEL_AUTO ){
      printf("L%05u:",label[addr]&0x3fff);
      size = 6;
    }else{
      printf("%s:",&label_buf[ label[addr]&0x7fff ]);
      size = strlen( &label_buf[ label[addr]&0x7fff ] );
    }
  }else{
    size = 0;
  }
  for( i=7; i<tab_width; i+=8 ){
    if( size<i ){
      for( j=0; j<((tab_width-i)/8)+1; j++ ) printf("\t");
      break;
    }
  }



}




void	disasm( void )
{
  int	addr  = disasm_start;
  int	count = disasm_size;
  int	d, num;
  char	buf[33];
  Mnemonics *Inst;

  while( 1 ){

    if      ( attr[addr] & 0x80 ){		/* DS */

      d = (attr[addr]&0x7f) +1;
      printf_head( addr, -d );
      printf(" DS	0%XH\n",d );
      addr += d;
      count-= d;

    }else if( attr[addr] & 0x60 ){		/* DB, DC, DW */

      switch( attr[addr] & 0x60 ){

      case ATTR_DB:
	d = (attr[addr]&0x1f) +1;
	while( d>0 ){
	  switch( d ){
	  case 1:
	    printf_head( addr, 1 );
	    printf(" DB	0%02XH\n", prog[addr] );
	    d=0; addr+=1; count-=1;
	    break;
	  case 2:
	    printf_head( addr, 2 );
	    printf(" DB	0%02XH,0%02XH\n", prog[addr], prog[addr+1] );
	    d=0; addr+=2; count-=2;
	    break;
	  case 3:
	    printf_head( addr, 3 );
	    printf(" DB	0%02XH,0%02XH,0%02XH\n",
		   prog[addr], prog[addr+1], prog[addr+2]);
	    d=0; addr+=3; count-=3;
	    break;
	  default:
	    printf_head( addr, 4 );
	    printf(" DB	0%02XH,0%02XH,0%02XH,0%02XH\n",
		   prog[addr], prog[addr+1], prog[addr+2], prog[addr+3]);
	    d-=4; addr+=4; count-=4;
	    break;
	  }
	}
	break;


      case ATTR_DC:
#if 0
	d = (attr[addr]&0x1f) +1;
	num = 0;
	while( d>0 ){
	  buf[ num ] = (prog[ addr+num ]<32 || 126<prog[ addr+num ]) ?
	                                                '_' : prog[ addr+num ];
	  num++;
	  d--;
	}
	buf[num]='\0';

	d = (attr[addr]&0x1f) +1;
	if( d>=4 ){ printf_head( addr, 4 ); d-=4; addr+=4; count-=4;      }
	else      { printf_head( addr, d );       addr+=d; count-=d; d=0; }
	printf(" DB	'%s'\n",buf);
	while( d>0 ){
	  if( d>=4 ){ printf_head( addr, 4 ); d-=4; addr+=4; count-=4;      }
	  else      { printf_head( addr, d );       addr+=d; count-=d; d=0; }
	  printf("\n");
	}
#else
	d = (attr[addr]&0x1f) +1;
	for( num=0; num<d; num++ ){
	  buf[ num ] = prog[ addr+num ];
	}
	if( d>=4 ){ printf_head( addr, 4 ); d-=4; addr+=4; count-=4;      }
	else      { printf_head( addr, d );       addr+=d; count-=d; d=0; }
	{
	  int after_str  = FALSE;
	  int first_byte = TRUE;
	  int i;
	  printf(" DB	");
	  for( i=0; i<num; i++ ){	/* num νͤ(attr[addr]&0x1f)+1 */
	    if( (unsigned char)buf[i]<32 || 126<(unsigned char)buf[i] ){
	      if( first_byte ) ;
	      else{
		if( after_str ) printf("',");
		else            printf( ",");
	      }
	      printf("0%02XH",buf[i]);
	      after_str = FALSE;
	    }else{
	      if( first_byte )  printf( "'");
	      else{
		if( after_str ) ;
		else            printf(",'");
	      }
	      printf("%c",buf[i]);
	      after_str = TRUE;
	    }
	    first_byte = FALSE;
	  }
	  if( after_str ) printf("'\n");
	  else            printf("\n");
	}
	while( d>0 ){
	  if( d>=4 ){ printf_head( addr, 4 ); d-=4; addr+=4; count-=4;      }
	  else      { printf_head( addr, d );       addr+=d; count-=d; d=0; }
	  printf("\n");
	}
#endif
	break;  


      case ATTR_DW:
	d = ((attr[addr]&0x1f) +1) *2;
	while( d>0 ){
	  switch( d ){
	  case 2:
	    printf_head( addr, 2 );
	    printf(" DW	0%04XH\n", prog[addr]+prog[addr+1]*256 );
	    d=0; addr+=2; count-=2;
	    break;
	  default:
	    printf_head( addr, 4 );
	    printf(" DW	0%04XH,0%04XH\n",
		   prog[addr]+prog[addr+1]*256, prog[addr+2]+prog[addr+3]*256);
	    d-=4; addr+=4; count-=4;
	    break;
	  }
	}
	break;

      }

    }else{

      Inst = &Instruction[ prog[addr] ];
      switch( Inst->type ){
      case OP_NOTHING:
	printf_head( addr, 1 );
	printf( Inst->str );
	printf("\n");
	d = 1;
	break;

      case OP_NUM_8:
	num = prog[addr+1];
	printf_head( addr, 2 );
	printf( Inst->str, num );
	printf("\n");
	d = 2;
	break;

      case OP_NUM_16:
	num = prog[addr+1] + prog[addr+2]*256;
	printf_head( addr, 3 );
	printf( Inst->str, num );
	printf("\n");
	d = 3;
	break;

      case OP_MEMORY:
	num = prog[addr+1] + prog[addr+2]*256;
	printf_head( addr, 3 );
	printf( Inst->str, num );
	printf("\n");
	d = 3;
	break;
	
      case OP_ADR_ABS:
	num = prog[addr+1] + prog[addr+2]*256;
	printf_head( addr, 3 );
	if((label[num]&0x8000)==LABEL_USER ){
	  printf( Inst->str, &label_buf[ label[num]&0x7fff ]);
	}else{
	  if( num<disasm_start || disasm_start+disasm_size<=num ){
	    sprintf( &buf[0], "0%04XH", num );
	    printf( Inst->str, buf );
	  }else{
	    buf[0]='L';
	    sprintf( &buf[1], "%05d", label[num]&0x3fff );
	    printf( Inst->str, buf );
	  }
	}
	printf("\n");
	d = 3;
	break;

      case OP_ADR_REL:
	num = addr +2 + (signed char)prog[addr+1];
	printf_head( addr, 2 );
	if((label[num]&0x8000)==LABEL_USER ){
	  printf( Inst->str, &label_buf[ label[num]&0x7fff ]);
	}else{
	  if( num<disasm_start || disasm_start+disasm_size<=num ){
	    sprintf( &buf[0], "0%04XH", num );
	    printf( Inst->str, buf );
	  }else{
	    buf[0]='L';
	    sprintf( &buf[1], "%05d", label[num]&0x3fff );
	    printf( Inst->str, buf );
	  }
	}
	printf("\n");
	d = 2;
	break;


      case OP_PREFIX:
	switch( prog[addr] ){
	case 0xCB:	Inst = &Instruction_CB[ prog[addr+1] ];	break;
	case 0xDD:	Inst = &Instruction_DD[ prog[addr+1] ];	break;
	case 0xED:	Inst = &Instruction_ED[ prog[addr+1] ];	break;
	case 0xFD:	Inst = &Instruction_FD[ prog[addr+1] ];	break;
	default:	fprintf(stderr,"Internal Error 4!!\n");	exit(2);
	}
	switch( Inst->type ){
	case OP_NOTHING:
	  printf_head( addr, 2 );
	  printf( Inst->str );
	  printf("\n");
	  d = 2;
	  break;

	case OP_UNEXIST:
	  printf_head( addr, 2 );
	  printf( Inst->str, prog[addr], prog[addr+1] );
	  printf("\n");
	  d = 2;
	  break;

	case OP_SKIP:
	  printf_head( addr, 1 );
	  printf( Inst->str, prog[addr] );
	  printf("\n");
	  d = 1;
	  break;

	case OP_NUM_8:
	  num = prog[addr+2];
	  printf_head( addr, 3 );
	  printf( Inst->str, num );
	  printf("\n");
	  d = 3;
	  break;

	case OP_NUM_16:
	  num = prog[addr+2] + prog[addr+3]*256;
	  printf_head( addr, 4 );
	  printf( Inst->str, num );
	  printf("\n");
	  d = 4;
	  break;

	case OP_MEMORY:
	  num = prog[addr+2] + prog[addr+3]*256;
	  printf_head( addr, 4 );
	  printf( Inst->str, num );
	  printf("\n");
	  d = 4;
	  break;
	
	case OP_INDEX:
	  printf_head( addr, 3 );
	  printf( Inst->str, (signed char)prog[addr+2] );
	  printf("\n");
	  d = 3;
	  break;

	case OP_IDX_NUM:
	  num = (int)prog[addr+3];
	  printf_head( addr, 4 );
	  printf( Inst->str, (int)prog[addr+2] ,num );
	  printf("\n");
	  d = 4;
	  break;


	case OP_PREFIX:
	  switch( prog[addr] ){
	  case 0xDD:	Inst = &Instruction_DD_CB[ prog[addr+3] ];	break;
	  case 0xFD:	Inst = &Instruction_FD_CB[ prog[addr+3] ];	break;
	  default:	fprintf(stderr,"Internal Error 5!!\n");	exit(2);
	  }
	  switch( Inst->type ){
	  case OP_INDEX:
	    printf_head( addr, 4 );
	    printf( Inst->str, (signed char)prog[addr+2] );
	    printf("\n");
	    d = 4;
	    break;

	  case OP_UNEXIST:
	    printf_head( addr, 4 );
	    printf( Inst->str,
		       prog[addr], prog[addr+1], prog[addr+2], prog[addr+3] );
	    printf("\n");
	    d = 4;
	    break;
	  }
	  break;
	}
	break;
      }

      addr  += d;
      count -= d;
    }




    if( count==0 ) return;
    if( addr>0xffff ){
      fprintf(stderr, "PC Overflow (>FFFFH)!?\n");
      exit(1);
    }
    if( count <0 ){
      fprintf(stderr, "size mismatch\n");
      exit(1);
    }
  }
}
